#############################################################################################
#####function randomly generate matrix based on given row(r) and column(c) number,
#####a is the species richness of survey sites to be referred 
#############################################################################################
ranGenMat <- function(r,c,a){
  m <- matrix(0, r, c)
  success <- FALSE
  while(!success){
    for(i in 1:length(a)){
      m[i,] <- sample(c(0,1),c,replace = T,
                      prob = c((1-a[i]/c),a[i]/c))}
    success <- (min(rowSums(m))!=0 | min(colSums(m))!=0) # matrix that no column sum nor row sum is zero will be generated
  }    
  return(m)
} 


#############################################################################################
######## function to get the subdiagonal values of matrix
#############################################################################################
matdiag <- function(x){
  x <- as.matrix(x)
  x <- x[-1,]
  n <- min(dim(x))
  return(x[matrix(rep(1:n, 2), n, 2)])
}
#############################################################################################
library(betapart)
library(vegan)
library(openxlsx)

####load the richness of different lifeforms in each sites to be referred in simulation

    rich <- read.xlsx("supplementary_table.xlsx",sheet = 1)
    rich <- rich[-c(1,2,27,28),]
    colnames(rich) <- c('sites','altitude','tree','shrub','herb','richness','dom_spes')
#############################################################################################
######create null lists or matrices for different lifeforms
#############################################################################################
    matlist.t <- list()
    matlist.s <- list()
    matlist.h <- list()
    matlist.a <- list()
    t.sim <- matrix(,23,1000)
    t.sor <- matrix(,23,1000)
    t.sne <- matrix(,23,1000)
    s.sim <- matrix(,23,1000)
    s.sor <- matrix(,23,1000)
    s.sne <- matrix(,23,1000)
    h.sim <- matrix(,23,1000)
    h.sor <- matrix(,23,1000)
    h.sne <- matrix(,23,1000)
    a.sim <- matrix(,23,1000)
    a.sor <- matrix(,23,1000)
    a.sne <- matrix(,23,1000)
    #############################################################################################
    #######start simulation 
    #############################################################################################
    for(i in 1:1000){ 
        matlist.t[[i]]<- ranGenMat(24,83,rich$t)
        matlist.s[[i]]<- ranGenMat(24,86,rich$s)
        matlist.h[[i]]<- ranGenMat(24,269,rich$h)
        matlist.a[[i]]<- cbind(matlist.t[[i]],matlist.s[[i]],matlist.h[[i]])
  
  #########tree##############################################################################
  t.sim[,i] <- (lapply(beta.pair(matlist.t[[i]],index.family = "sorensen"),matdiag))$beta.sim
  t.sor[,i] <- (lapply(beta.pair(matlist.t[[i]],index.family = "sorensen"),matdiag))$beta.sor
  t.sne[,i] <- (lapply(beta.pair(matlist.t[[i]],index.family = "sorensen"),matdiag))$beta.sne
  #########shrub##############################################################################
  s.sim[,i] <- (lapply(beta.pair(matlist.s[[i]],index.family = "sorensen"),matdiag))$beta.sim
  s.sor[,i] <- (lapply(beta.pair(matlist.s[[i]],index.family = "sorensen"),matdiag))$beta.sor
  s.sne[,i] <- (lapply(beta.pair(matlist.s[[i]],index.family = "sorensen"),matdiag))$beta.sne
  #########herb##############################################################################
  h.sim[,i] <- (lapply(beta.pair(matlist.h[[i]],index.family = "sorensen"),matdiag))$beta.sim
  h.sor[,i] <- (lapply(beta.pair(matlist.h[[i]],index.family = "sorensen"),matdiag))$beta.sor
  h.sne[,i] <- (lapply(beta.pair(matlist.h[[i]],index.family = "sorensen"),matdiag))$beta.sne
  #########overall##############################################################################
  a.sim[,i] <- (lapply(beta.pair(matlist.a[[i]],index.family = "sorensen"),matdiag))$beta.sim
  a.sor[,i] <- (lapply(beta.pair(matlist.a[[i]],index.family = "sorensen"),matdiag))$beta.sor
  a.sne[,i] <- (lapply(beta.pair(matlist.a[[i]],index.family = "sorensen"),matdiag))$beta.sne
  
}
    #############################################################################################
    #####check the richness of each simulation
    #############################################################################################
    richness.t <- sapply(matlist.t,specnumber)
    richness.s <- sapply(matlist.s,specnumber)
    richness.h <- sapply(matlist.h,specnumber)
    richness.a <- sapply(matlist.a,specnumber)
        richness <- data.frame(apply(richness.t,1,mean),apply(richness.s,1,mean),
                               apply(richness.h,1,mean),apply(richness.a,1,mean))
        richness <- round(richness,digits = 0)
        colnames(richness) <- c('t','s','h','a')
  ##############################################################################
  ###get the simulated dissimilarity, turnover, nestedness for different lifeforms
  ##############################################################################
  simu.t <- data.frame(apply(t.sim,1,mean),apply(t.sor,1,mean),apply(t.sne,1,mean))
  simu.s <- data.frame(apply(s.sim,1,mean),apply(s.sor,1,mean),apply(s.sne,1,mean))
  simu.h <- data.frame(apply(h.sim,1,mean),apply(h.sor,1,mean),apply(h.sne,1,mean))
  simu.a <- data.frame(apply(a.sim,1,mean),apply(a.sor,1,mean),apply(a.sne,1,mean))

    simu <- data.frame(simu.t,simu.s,simu.h,simu.a)
    
          colnames(simu) <- c('tt','tb','tn','st','sb','sn','ht','hb','hn','at','ab','an')
    
 